module OfflineDeployer
  class Updater
    def initialize(zip_file)
      @zip_file = zip_file
    end

    attr_reader :zip_file

    EXTRACT_DIRECTORY = 'tmp/releases/updater'

    def call
      validate_zip_file
      remove_old_update_files
      extract_new_version
      validate_version
      update_changed_files
      remove_deleted_files
      run_update_script
      finish_and_restart_server
    end

    private

    def validate_zip_file
      fail 'Invalid file' if File.extname(zip_file.original_filename) != '.zip'
    end

    def remove_old_update_files
      FileUtils.remove_dir(EXTRACT_DIRECTORY) if File.exist?(EXTRACT_DIRECTORY)
    end

    def extract_new_version
      `echo 'Extracting zip' >> log/update.log`
      Archive::Zip.extract(zip_file.path, EXTRACT_DIRECTORY)
    end

    def old_version_info
      YAML.load_file('.update')
    end

    def new_version_info
      YAML.load_file(File.join(EXTRACT_DIRECTORY, '.update'))
    end

    def validate_version
      return if old_version_info[:TO_VERSION] == new_version_info[:FROM_VERSION]
      `echo 'Wrong version' >> log/update.log`
      fail 'Wrong version'
    end

    def update_changed_files
      `rsync -r -v #{EXTRACT_DIRECTORY}/* ./ >> log/update.log`
    end

    def remove_deleted_files
      deleted_files = new_version_info[:DELETED_FILES]
      return if deleted_files.nil?
      deleted_files.each { |deleted_file| `rm #{deleted_file}` }
    end

    def run_update_script
      `./update_script.sh`
      errors = File.open('log/update_error.log').read
      fail 'There were some errors while running the script!' if errors.present?
    end

    def finish_and_restart_server
      `cp #{EXTRACT_DIRECTORY}/.update ./.update`
      `#{OfflineDeployer.settings.restart_server_command}`
      `echo 'Server restarted' >> log/update.log`
    end
  end
end
